#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for authenticating to the Awair Developer API.
For more info on the Awair product line, please visit
https://www.getawair.com
"""

# This section imports required libraries


HEADERS = {
    'Accept': 'application/json', 'Content-Type':
        'application/json', 'Accept-encoding': 'application/json'
}


class AwairAuth:
    """
    Object to hold the authentication data for the Awair API
    :return An object of class AwairAuth to be passed into other functions to
    pass the authentication credentials
    """

    def __init__(self, token):
        """
        This class acts as the auth object for the Awair API. The token is available from the
        Awair developer website.
        :param token: str object which contains the
        """

        self.token = token
        self.headers = {
            'Accept': 'application/json', 'Content-Type':
                'application/json', 'Accept-encoding': 'application/json',
            'Authorization': 'Bearer {'
                             '}'.format(
                    self.token)
        }



from pyawair.devices import *


def get_organizations(auth):
    response = requests.get("http://developer-apis.awair.is/v1/org-users/self/orgs",
                            headers=auth.headers)
    return "Function not implemented yet"



from pyawair.devices import *
import pyawair.conn
import pyawair.objects


def get_current_air_data(auth, device_name=None, device_type=None, device_id=None):
    """
    Function to get the current air data from a single specific Awair Device linked
    to your account
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :param device_type: str which matches the awair device type
    :param device_name: str which matches exactly to the name of a specific device
    :param device_id: str or int which matches the specific awair device internal id number
    :return: Object of Dict type which contains current air data
    """
    if device_type is None or device_id is None:
        awair_device = pyawair.objects.AwairDev(device_name, auth)
        device_type = awair_device.type()
        device_id = awair_device.id()

    base_url = "http://developer-apis.awair.is/v1/users/self/devices/"
    data_url = "/air-data/latest"
    data = pyawair.conn.get_data(auth, device_id, device_type, base_url, data_url)
    return data


def get_5_min_average(auth, device_name=None, device_type=None, device_id=None, limit=1000,
                      desc=True):
    """
    Function to get air data that is averaged over each 5 minutes from a single specific Awair
    Device linked
    to your account
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :param device_type: str which matches the awair device type
    :param device_name: str which matches exactly to the name of a specific device
    :param device_id: str or int which matches the specific awair device internal id number
    :param limit: int that specifies the number of 5-minute periods to query
    :param desc: bool where True specifies descending (newest first) and False specifies
    ascending (oldest first)
    :return: Object of Dict type which contains current air data
    """
    if device_type is None or device_id is None:
        awair_device = pyawair.objects.AwairDev(device_name, auth)
        device_type = awair_device.type()
        device_id = awair_device.id()

    base_url = "http://developer-apis.awair.is/v1/users/self/devices/"
    data_url = "/air-data/5-min-avg"
    if desc:
        desc_param = "true"
    else:
        desc_param = "false"
    args = "?limit={}&desc={}".format(limit, desc_param)
    data = pyawair.conn.get_data(auth, device_id, device_type, base_url, data_url, args)

    return data


def get_15_min_average(auth, device_name=None, device_type=None, device_id=None, limit=1000,
                       desc=True):
    """
    Function to get air data that is averaged over each 5 minutes from a single specific Awair
    Device linked
    to your account
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :param device_type: str which matches the awair device type
    :param device_name: str which matches exactly to the name of a specific device
    :param device_id: str or int which matches the specific awair device internal id number
    :param limit: int that specifies the number of 15-minute periods to query
    :param desc: bool where True specifies descending (newest first) and False specifies
    ascending (oldest first)
    :return: Object of Dict type which contains current air data
    """
    if device_type is None or device_id is None:
        awair_device = pyawair.objects.AwairDev(device_name, auth)
        device_type = awair_device.type()
        device_id = awair_device.id()

    base_url = "http://developer-apis.awair.is/v1/users/self/devices/"
    data_url = "/air-data/15-min-avg"
    if desc:
        desc_param = "true"
    else:
        desc_param = "false"
    args = "?limit={}&desc={}".format(limit, desc_param)
    data = pyawair.conn.get_data(auth, device_id, device_type, base_url, data_url, args)

    return data


def get_raw_data(auth, device_name=None, device_type=None, device_id=None):
    """
        Function to get air data that is averaged over each 5 minutes from a single specific
        Awair Device linked
        to your account
        :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
        :param device_type: str which matches the awair device type
        :param device_name: str which matches exactly to the name of a specific device
        :param device_id: str or int which matches the specific awair device internal id number
        :return: Object of Dict type which contains current air data
        """
    if device_type is None or device_id is None:
        awair_device = pyawair.objects.AwairDev(device_name, auth)
        device_type = awair_device.type()
        device_id = awair_device.id()

    base_url = "http://developer-apis.awair.is/v1/users/self/devices/"
    data_url = "/air-data/raw"
    data = pyawair.conn.get_data(auth, device_id, device_type, base_url, data_url)

    return data



# !/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-

import pyawair.data
from pyawair.auth import AwairAuth
import datetime


class AwairDev:
    def __init__(self, device_name: str, auth: AwairAuth, cache_time: float = 15,
                 aggregate_type='15-minute'):
        """
        Initialise AwairDev object.

        :param device_name: The name of the device a can be found in the Awair app. Careful, case sensitive.
        :param auth: The authentication object as created by the AwairAuth function.
        :param cache_time: The time in minutes that the state values should be cached. When this time has expired, new values
                           will be requested. Keep in mind that the API has daily limits so setting this too low might
                           cause problems.
        :param aggregate_type: Type can be 'current', '5-minute' or '15-minute' referring to the aggregation. Keep in mind that
                               not all tiers have access to all of them.
        """
        self._auth = auth
        self._cache_time = cache_time
        if aggregate_type in ['current', '5-minute', '15-minute']:
            self._aggregate_type = aggregate_type
        else:
            raise ValueError("The argument aggregate_type cannot have this value.")
        self._last_update = datetime.datetime.now()  # records the last update

        self._device_name = device_name

        # Get device type and ID from name
        devices = pyawair.data.get_all_devices(self._auth)
        device_names = [d['name'] for d in devices]
        if not device_name in device_names:
            raise ValueError(
                "This device name ({}) does not exist in your Awair account. Be aware that the device name is capital sensitive.".format(
                    device_name))

        self._type = next((item for item in devices if item["name"] == device_name),
                          False)['deviceType']  # get the device type
        self._id = next((item for item in devices if item["name"] == device_name),
                        False)['deviceId']  # get the device ID

        # Initiate data fields
        self._data = {}
        self._last_update = None

        self.refresh()

    def get_state(self, indicator: str) -> float:
        """
        Function to get the state of a specific indicator.

        The values are cached, in accordance with the cache time that is set for the object.

        :param indicator: A string containing one of the values from: score, temp, humid, co2, voc or dust.
        :return: The value of the specific indicator.
        """
        now = datetime.datetime.now()
        delta_min = (now - self._last_update).total_seconds() / 60
        if delta_min > self._cache_time:
            self.refresh()
        return self._data[indicator]

    def name(self) -> str:
        """
        Function to get the name of the device.

        :return: The name of the device.
        """
        return self._device_name

    def type(self) -> str:
        """
        Function to get the name of the device.

        :return: The type of the device.
        """
        return self._type

    def id(self) -> str:
        """
        Function to get the name of the device.

        :return: The name of the device.
        """
        return self._id

    def refresh(self):
        """
        Function to refresh the state of the objects.

        The values are cached internally for the period equal to the cache
        time value. The refresh function refreshed these values, independent of the time that has past since the last
        refresh.
        """
        if self._aggregate_type == 'current':
            data: list = pyawair.data.get_current_air_data(self._auth, device_id=self._id,
                                                           device_type=self._type)
        elif self._aggregate_type == '5-minute':
            data: list = pyawair.data.get_5_min_average(self._auth, device_id=self._id,
                                                        device_type=self._type)
        elif self._aggregate_type == '15-minute':
            data: list = pyawair.data.get_15_min_average(self._auth, device_id=self._id,
                                                         device_type=self._type)

        self._data['score'] = data[-1]['score']
        self._data['temp'] = data[-1]['sensors'][0]['value']
        self._data['humid'] = data[-1]['sensors'][1]['value']
        self._data['co2'] = data[-1]['sensors'][2]['value']
        self._data['voc'] = data[-1]['sensors'][3]['value']
        if self._type != 'awair-glow':   #Glow doesn't have dust sensor so failing
            self._data['dust'] = data[-1]['sensors'][4]['value']
        self._last_update = datetime.datetime.now()  # records the time of the last update



import json
import requests
import pyawair.conn


def get_user_data(auth):
    """
    Function to get the user data for the account linked to the token
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :return: Object of Dict type which contains current user data for this account
    """
    response = requests.get("http://developer-apis.awair.is/v1/users/self",
                            headers=auth.headers)
    return json.loads(response.text)


def get_all_devices(auth):
    """
    Function to get all devices for the account linked to the token
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :return: Object of Dict type which contains a list of all devices for this account
    """
    response = requests.get("http://developer-apis.awair.is/v1/users/self/devices",
                            headers=auth.headers)
    pyawair.conn.check_response(response)
    return json.loads(response.text)['devices']


def get_dev_details(auth, device_name):
    """
    Function to get the data for a single specific device for the account linked
    to the token
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :param device_name: str which matches exactly to the name of a specific device
    :return: Object of Dict type  which contains device details for specific device
    """
    devices = get_all_devices(auth)
    for dev in devices:
        if dev['name'] == device_name:
            return dev
    return "Device not found"


def get_dev_led_mode(auth, device_name=None, device_type=None, device_id=None):
    """
    Function to get the LED mode for a single specific devices for the account
    linked to the token
    :param device_type:
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :param device_name: str which matches exactly to the name of a specific device
    :param device_name: str which matches exactly to the name of a specific device
    :param device_id: str or int which matches the specific awair device internal id number
    :return: Object of Dict type which describes the LED mode of the specified devices
    """
    if device_name is None:
        base_url = "http://developer-apis.awair.is/v1/devices/"
        dev_url = device_type + "/" + str(device_id)
        data_url = "/led"
        f_url = base_url + dev_url + data_url
        response = requests.get(f_url, headers=auth.headers)
        pyawair.conn.check_response(response)
        return json.loads(response.text)
    elif device_name is not None:
        devices = get_all_devices(auth)
        for dev in devices:
            if dev['name'] == device_name:
                base_url = "http://developer-apis.awair.is/v1/devices/"
                dev_url = dev['deviceType'] + "/" + str(dev['deviceId'])
                data_url = "/led"
                f_url = base_url + dev_url + data_url
                response = requests.get(f_url, headers=auth.headers)
                pyawair.conn.check_response(response)
                return json.loads(response.text)
            else:
                return "Device not found"


def get_dev_timezone(auth, device_name=None, device_type=None, device_id=None):
    """
    Function to get the timezone for a single specific devices for the account
    linked to the token. Refer to column TZ in
    https://en.wikipedia.org/wiki/List_of_tz_database_time_zones
    :param device_type:
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :param device_name: str which matches exactly to the name of a specific device
    :param device_name: str which matches exactly to the name of a specific device
    :param device_id: str or int which matches the specific awair device internal id number
    :return: Object of Dict type which describes the timezone of the specified devices
    """
    if device_name is None:
        base_url = "http://developer-apis.awair.is/v1/devices/"
        dev_url = device_type + "/" + str(device_id)
        data_url = "/timezone"
        f_url = base_url + dev_url + data_url
        response = requests.get(f_url, headers=auth.headers)
        pyawair.conn.check_response(response)
        return json.loads(response.text)
    else:
        devices = get_all_devices(auth)
        for dev in devices:
            if dev['name'] == device_name:
                base_url = "http://developer-apis.awair.is/v1/devices/"
                dev_url = dev['deviceType'] + "/" + str(dev['deviceId'])
                data_url = "/timezone"
                f_url = base_url + dev_url + data_url
                response = requests.get(f_url, headers=auth.headers)
                pyawair.conn.check_response(response)
                return json.loads(response.text)
            else:
                return "Device not found"


def get_dev_display_mode(auth, device_name=None, device_type=None, device_id=None):
    """
    Function to get the display mode for a single specific devices for the account
    linked to the token
    :param device_type:
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :param device_name: str which matches exactly to the name of a specific device
    :param device_name: str which matches exactly to the name of a specific device
    :param device_id: str or int which matches the specific awair device internal id number
    :return: Object of Dict type which describes the display mode of the specified devices
    """
    if device_name is None:
        base_url = "http://developer-apis.awair.is/v1/devices/"
        dev_url = device_type + "/" + str(device_id)
        data_url = "/display"
        f_url = base_url + dev_url + data_url
        response = requests.get(f_url, headers=auth.headers)
        pyawair.conn.check_response(response)
        return json.loads(response.text)
    else:
        devices = get_all_devices(auth)
        for dev in devices:
            if dev['name'] == device_name:
                base_url = "http://developer-apis.awair.is/v1/devices/"
                dev_url = dev['deviceType'] + "/" + str(dev['deviceId'])
                data_url = "/display"
                f_url = base_url + dev_url + data_url
                response = requests.get(f_url, headers=auth.headers)
                pyawair.conn.check_response(response)
                return json.loads(response.text)
            else:
                return "Device not found"


def get_dev_power_status(auth, device_name=None, device_type=None, device_id=None):
    """
    Function to get the power_status for a single specific devices for the account
    linked to the token.
    :param device_type:
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :param device_name: str which matches exactly to the name of a specific device
    :param device_name: str which matches exactly to the name of a specific device
    :param device_id: str or int which matches the specific awair device internal id number
    :return: Object of Dict type which describes the power status of the specified devices
    """
    if device_name is None:
        base_url = "http://developer-apis.awair.is/v1/devices/"
        dev_url = device_type + "/" + str(device_id)
        data_url = "/power-status"
        f_url = base_url + dev_url + data_url
        response = requests.get(f_url, headers=auth.headers)
        pyawair.conn.check_response(response)
        return json.loads(response.text)
    else:
        devices = get_all_devices(auth)
        for dev in devices:
            if dev['name'] == device_name:
                base_url = "http://developer-apis.awair.is/v1/devices/"
                dev_url = dev['deviceType'] + "/" + str(dev['deviceId'])
                data_url = "/power-status"
                f_url = base_url + dev_url + data_url
                response = requests.get(f_url, headers=auth.headers)
                pyawair.conn.check_response(response)
                return json.loads(response.text)
            else:
                return "Device not found"


"""Following section deals with changing settings on the Awair devices"""


def set_device_preference(auth, new_mode, device_name=None, device_type=None, device_id=None):
    """
    Function to set the prefered mode for a single specific devices for the account
    linked to the token.
    :param device_type:
    :param auth: pyawair.auth.AwairAuth object which contains a valid authentication token
    :param new_mode: str which matches the desired preference mode. Valid modes are
    "general", "productivity", "sleep", "allergy", "baby"
    :param device_name: str which matches exactly to the name of a specific device
    :param device_name: str which matches exactly to the name of a specific device
    :param device_id: str or int which matches the specific awair device internal id number
    :return: Object of Dict type which contains a message of whether or not the set request was
    implemented
    """
    if device_name is None:
        base_url = "http://developer-apis.awair.is/v1/devices/"
        dev_url = device_type + "/" + str(device_id)
        data_url = "/preference"
        f_url = base_url + dev_url + data_url
        modes = ['general', 'productivity', 'sleep', 'allergy', 'baby']
        new_mode = new_mode
        if new_mode in modes:
            data = json.dumps({'pref': new_mode})
            response = requests.put(f_url, data=data, headers=auth.headers)
            pyawair.conn.check_response(response)
            return json.loads(response.text)
        else:
            return ("mode setting not valid")
    else:
        devices = get_all_devices(auth)
        for dev in devices:
            if dev['name'] == device_name:
                base_url = "http://developer-apis.awair.is/v1/devices/"
                dev_url = dev['deviceType'] + "/" + str(dev['deviceId'])
                data_url = "/preference"
                f_url = base_url + dev_url + data_url
                modes = ['general', 'productivity', 'sleep', 'allergy', 'baby']
                mode = new_mode
                if mode in modes:
                    data = json.dumps({'pref': new_mode})
                    response = requests.put(f_url, data=data, headers=auth.headers)
                    pyawair.conn.check_response(response)
                    return json.loads(response.text)
                else:
                    return ("mode setting not valid")
            else:
                return "Device not found"


def set_device_timezone(auth, timezone, device_name=None, device_type=None, device_id=None):
    """
    This function takes a str input for the value timezone and applies it to a specific
    awair device. The device can be selected using the unique name of the device, or by
    using a combination of the device type and device id values specific to the API.
    :param auth:
    :param timezone: str value of the desired timezone.
    For specific values please see
    https://en.wikipedia.org/wiki/List_of_tz_database_time_zones
    :param device_name: str value of the name of the device
    :param device_type: str value of the device_type
    :param device_id: int or str value of the device_id (internal API id)
    :return: dictionary


    new_timezone = "America/Montreal"
    x = set_device_timezone(new_timezone, device_name="Bedroom_Awair")

    """
    if device_name is None:
        base_url = "http://developer-apis.awair.is/v1/devices/"
        dev_url = device_type + "/" + str(device_id)
        data_url = "/timezone"
        f_url = base_url + dev_url + data_url
        data = json.dumps({'timezone': timezone})
        response = requests.post(f_url, data=data, headers=auth.headers)
        pyawair.conn.check_response(response)
        return json.loads(response.text)
    else:
        devices = get_all_devices(auth)
        for dev in devices:
            if dev['name'] == device_name:
                base_url = "http://developer-apis.awair.is/v1/devices/"
                dev_url = dev['deviceType'] + "/" + str(dev['deviceId'])
                data_url = "/timezone"
                f_url = base_url + dev_url + data_url
                data = json.dumps({'timezone': timezone})
                response = requests.post(f_url, data=data, headers=auth.headers)
                pyawair.conn.check_response(response)
                return json.loads(response.text)
            else:
                return "Device not found"


def set_device_led(auth, led_mode, device_name=None, device_type=None, device_id=None):
    """
    This function takes a str input for the value timezone and applies it to a specific
    awair device. The device can be selected using the unique name of the device, or by
    using a combination of the device type and device id values specific to the API.
    :param auth:
    :param led_mode: str value of the desired LED mode. Valid inputs are:
    "on" "dim" "sleep
    :param device_name: str value of the name of the device
    :param device_type: str value of the device_type
    :param device_id: int or str value of the device_id (internal API id)
    :return: dictionary


    new_led_mode = "dim"
    x = set_device_timezone(new_timezone, device_name="Bedroom_Awair")

    """
    modes = "on, dim, sleep"
    if device_name is None:
        base_url = "http://developer-apis.awair.is/v1/devices/"
        dev_url = device_type + "/" + str(device_id)
        data_url = "/led"
        f_url = base_url + dev_url + data_url
        if led_mode in modes:
            data = json.dumps({'mode': led_mode})
            response = requests.post(f_url, data=data, headers=auth.headers)
            pyawair.conn.check_response(response)
            return json.loads(response.text)
        else:
            return "mode setting not valid"
    else:
        devices = get_all_devices(auth)
        for dev in devices:
            if dev['name'] == device_name:
                base_url = "http://developer-apis.awair.is/v1/devices/"
                dev_url = dev['deviceType'] + "/" + str(dev['deviceId'])
                data_url = "/led"
                f_url = base_url + dev_url + data_url
                if led_mode in modes:
                        data = json.dumps({'mode': led_mode})
                        response = requests.post(f_url, data=data, headers=auth.headers)
                        pyawair.conn.check_response(response)
                        return json.loads(response.text)
                else:
                    return "mode setting not valid"
            else:
                return "Device not found"





import requests
import ast
import json



def check_response(response):
    if response.status_code != 200:
        raise ConnectionError("No connection with the API. Status code {}. Message: '{}'.".format(
            response.status_code,
            ast.literal_eval(response.text)['message']))
    return


def get_data(auth, id, type, base_url, data_url, args=''):
    dev_url = type + "/" + str(id)
    f_url = base_url + dev_url + data_url + args
    response = requests.get(f_url, headers=auth.headers)
    check_response(response)
    return json.loads(response.text)['data']
